library combine_latest;

import 'dart:async';
import 'package:composite_subscription/composite_subscription.dart';

class CombineLatest<S, T> extends StreamEventTransformer {

  final List<Stream<S>> _streams;
  final Function _selector;

  const CombineLatest(this._streams, this._selector);

  Stream<T> bind(Stream<S> stream) {
    var controller    = new StreamController();
    var seen          = new List.filled(this._streams.length + 1, false);
    var latests       = new List.filled(this._streams.length + 1, false);
    var subscriptions = null;
    var counter       = 0;
    var streams       = new List();
    streams.add(stream);
    streams.addAll(this._streams);

    subscriptions = streams.fold(
      new CompositeSubscription(),
      (composite, subject) {
        var id = counter++;
        return composite.add(subject.listen(
          (S data) {
            seen[id]    = true;
            latests[id] = data;
            if (seen.every((n) => n)) {
              controller.add(this._selector(latests));
            }
          },
          onError: (error) {
            controller.addError(error);
            subscriptions.cancel();
          },
          onDone: () {
            controller.close();
            subscriptions.cancel();
          }
        ));
      }
    );

    return controller.stream;
  }

}